@@ -162,7 +162,7 @@ dlep_base_proto_radio_init(void) {
 
 static void
 _cb_init_radio(struct dlep_session *session) {
-  if (session->next_signal == DLEP_PEER_INITIALIZATION) {
+  if (session->restrict_signal == DLEP_PEER_INITIALIZATION) {
     /*
      * we are waiting for a Peer Init,
      */
@@ -182,7 +182,7 @@ static int
 _radio_process_peer_discovery(
     struct dlep_extension *ext __attribute__((unused)),
     struct dlep_session *session) {
-  if (session->next_signal != DLEP_PEER_DISCOVERY) {
+  if (session->restrict_signal != DLEP_PEER_DISCOVERY) {
     /* ignore unless we are in discovery mode */
     return 0;
   }
@@ -199,7 +199,7 @@ _radio_process_peer_init(
   struct dlep_parser_value *value;
   const uint8_t *ptr;
 
-  if (session->next_signal != DLEP_PEER_INITIALIZATION) {
+  if (session->restrict_signal != DLEP_PEER_INITIALIZATION) {
     /* ignore unless we are in initialization mode */
     return 0;
   }
@@ -207,7 +207,7 @@ _radio_process_peer_init(
   /* mandatory heartbeat tlv */
   if (dlep_reader_heartbeat_tlv(
       &session->remote_heartbeat_interval, session, NULL)) {
-    OONF_WARN(session->log_source, "no heartbeat tlv, should not happen!");
+    OONF_INFO(session->log_source, "no heartbeat tlv");
     return -1;
   }
 
@@ -251,7 +251,7 @@ _radio_process_peer_init(
     }
   }
 
-  session->next_signal = DLEP_ALL_SIGNALS;
+  session->next_restrict_signal = DLEP_ALL_SIGNALS;
 
   return 0;
 }
@@ -278,6 +278,7 @@ _radio_process_destination_up(
     struct dlep_session *session) {
   struct netaddr mac;
   if (dlep_reader_mac_tlv(&mac, session, NULL)) {
+    OONF_INFO(session->log_source, "No mac TLV found");
     return -1;
   }
 
@@ -333,6 +334,7 @@ _radio_process_destination_down_ack(
   struct dlep_local_neighbor *local;
   struct netaddr mac;
   if (dlep_reader_mac_tlv(&mac, session, NULL)) {
+    OONF_INFO(session->log_source, "Could not read MAC tlv");
     return -1;
   }
 
